#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov  2 11:58:46 2020

@author: arnau
"""

from os import listdir, makedirs
from os.path import isfile, join, exists
import numpy as np
import pandas as pd
from csv import reader, writer
from numpy import f2py
import subprocess
import argparse
import sys

print('STARTING')
#Function to introduce boolean arguments 
def str2bool(v):
    if isinstance(v, bool):
       return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')

print('PARSING ARGUMENTS')
def getOptions(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(description="Parses command.")
    parser.add_argument("-d", "--datadir", help="Where the original data is stored.")
    parser.add_argument("-s", "--stationsdir", help="Where the info about the stations is stored.")
    parser.add_argument("-u", "--unitchange", help="Where the file with the conversion factors is stored.")
    parser.add_argument("-e", "--element", nargs = '+', help="Which element to process.")
    #if fortran script is not compiled uncomment next line and correspondent block code
    # parser.add_argument("-c", "--compile", help="Where the code to compile is.")
    parser.add_argument("-x", "--execute", help="Where the fortran code (rotate) to execute is.")
    parser.add_argument("-o", "--obsnorotated", help="Where to save the no rotated observations.")
    parser.add_argument("-f", "--obsrotated", help="Where to save the rotated observations.")
    parser.add_argument("-a", "--saveAllinOne", type=str2bool, nargs='?', const=True, default=False,
                        help="Save final obs separated by elements or all in one file.")
    options = parser.parse_args(args)
    return options

options = getOptions(sys.argv[1:])

# dataDir = '/home/arnau/Escritorio/DADES-ARNAU/CURRO/BSC-CNS/CAMS50/2018/data/data.background.assimilation/prova/'
dataDir = options.datadir

# stationsDir = '/home/arnau/Escritorio/DADES-ARNAU/CURRO/BSC-CNS/CAMS50/2018/'
stationsDir = options.stationsdir

# unitChangeFile = '/home/arnau/Escritorio/DADES-ARNAU/CURRO/BSC-CNS/CAMS50/2018/unit_change'
unitChangeFile = options.unitchange

# elements = ['CO', 'NO2', 'O3', 'PM10', 'PM25', 'SO2']
elements = options.element

# obs_no_rotated_save = '/home/arnau/Escritorio/DADES-ARNAU/CURRO/BSC-CNS/CAMS50/2018/obs/'
obs_no_rotated_save = options.obsnorotated
if not exists(obs_no_rotated_save):
    makedirs(obs_no_rotated_save)

# obs_final_save = '/home/arnau/Escritorio/DADES-ARNAU/CURRO/BSC-CNS/CAMS50/2018/obsfinal/'
obs_final_save = options.obsrotated
if not exists(obs_final_save):
    makedirs(obs_final_save)
    
# fortranscript = '/home/arnau/Escritorio/DADES-ARNAU/CURRO/BSC-CNS/CAMS50/scripts/rotated.f90'
# fortranexec = '/home/arnau/Escritorio/DADES-ARNAU/CURRO/BSC-CNS/CAMS50/scripts/rotated.exe'
fortranexec = options.execute

saveallinone = options.saveAllinOne

#IDs in the DA code
id_data = {'in_situ':90, 'O3':91, 'NO2':92, 'SO2':93, 'CO':94, 'PM10':95, 'PM2p5':96}
aidi = pd.Series(id_data)

#read unit change file
unit = pd.read_csv(unitChangeFile, sep = ' ', index_col = 0, header = None).T
#create dataframe with the dates
iDate = '2018-01-01'
fDate = '2019-01-01'
lafrec = 'H'
dates = pd.date_range(iDate, fDate, freq = lafrec, closed = 'left').strftime('%Y%m%d%H')

for e in elements:
    print(str(e))

print('READING STATIONS INFO ')
#Read the information about the stations
stationsasi = dict()
stationsvali = dict()
statHeader = ['statName', 'statLon', 'statLat', 'statNumber', 'Description1', 'Description2']
for e in elements:
    stationsasi[str(e)] = pd.read_csv(stationsDir + 'stations.' + str(e) + '.background_assimilation_set10',
                                   sep = ' ', names = statHeader)
    # stationsvali[str(e)] = pd.read_csv(stationsDir + 'stations.' + str(e) + '.background_validation_set10',
                                   # sep = ' ', names = statHeader)

obs = dict()
obsnorot = dict()
obsHeader = ['obstype', 'obslon', 'obslat', 'obsn', 'obsid', 'obslev', 'obs', 'obserr']
obsnorotHeader = ['date', 'obslon', 'obslat', 'obs', 'obserr']
for e in elements:
    obsnorot[str(e)]=dict()
    for d in dates:
        # obs[str(d)] = pd.DataFrame(columns = obsHeader)
        obsnorot[str(e)][str(d)] = pd.DataFrame(columns = obsnorotHeader)
print('READING DATA FILES')
#read the data files
for e in elements:
    dataDirElement = join(dataDir, str(e))
    for f in listdir(dataDirElement):
        file = join(dataDirElement,f)
        if isfile(file):
            statioName = f.split('.')[0]
            statioLon = stationsasi[str(e)].loc[stationsasi[str(e)]['statName'] == statioName, 'statLon'].values
            statioLat = stationsasi[str(e)].loc[stationsasi[str(e)]['statName'] == statioName, 'statLat'].values
            
            #next station if this is not in the assimilation list
            if len(statioLon) == 0:
                continue
            #This error will appear if some stations have various coordinates associated
            if len(statioLon) > 1:
                raise Exception('Caution, station {} has more than 1 longitudes associated'.format(statioName))
            elif len(statioLat) > 1:
                raise Exception('Caution, station {} has more than 1 latitudes associated'.format(statioName))
                
            # open the data file and iterate through its rows
            with open(file, 'r') as read_obj:
                csv_reader = reader(read_obj, delimiter = ' ')
                for row in csv_reader:
                    newdatarow = {'date':str(row[0]),'obslon':statioLon[0], 'obslat':statioLat[0]
                                  , 'obs':float(row[1])*eval(unit[e][1])
                                  , 'obserr':1}
                    obsnorot[str(e)][row[0]] = obsnorot[str(e)][row[0]].append(newdatarow, ignore_index = True)
#%% 
print('SAVING NO ROTATED OBSERVATIONS')                   
#save 
for e in elements:
    obsDirElement = join(obs_no_rotated_save, str(e))
    if not exists(obsDirElement):
        makedirs(obsDirElement)
    for key in obsnorot[str(e)]:
        obsfile = open(obsDirElement + '/' + str(key) + '_natural_glo.txt','w')
        obsfile.write(str(len(obsnorot[str(e)][key]))+'\n')
        obsnorot[str(e)][key].to_csv(obsfile, sep = ',', index=False, header=False)
        obsfile.close()

#%%
#RUN IF FORTRAN CODE IS NOT COMPILED

# with open(fortranscript) as sourcefile:
    # sourcecode = sourcefile.read()
# f2py.compile(sourcecode, modulename='rotar')
#%%
#Run fortran script (ROTATE)
print('ROTATING')
for e in elements:
    obsDirElement = join(obs_no_rotated_save, str(e))
    for key in obsnorot[str(e)]:
        entrada = obsDirElement + '/' + str(key) + '_natural_glo.txt'
        sortida1 = obsDirElement + '/' + str(key) + '_rotated_reg.txt'
        sortida2 = obsDirElement + '/' + str(key) + '_natural_reg.txt'
        subprocess.run([fortranexec, entrada, sortida1, sortida2])
#%%        
#Write final obs in DA binary format
print('WRITING IN DA BINARY FORMAT')
for e in elements:
    obsDirElement = join(obs_no_rotated_save, str(e))
    #if the user wants to save all obs in the same file 
    if saveallinone:
        obsfinalDirElement = obs_final_save
    else:
        obsfinalDirElement = join(obs_final_save, str(e))
        
    if not exists(obsfinalDirElement):
        makedirs(obsfinalDirElement)          
    for key in obsnorot[str(e)]:
        fileread = obsDirElement + '/' + str(key) + '_rotated_reg.txt'
        filewrite = obsfinalDirElement + '/obs' + str(key)

        pandaread = pd.read_csv(fileread, sep = '\s+', header = None, engine='python', dtype=str)
        datawrite = {'obstype':'obstype='+str(90), 'obslon':'obslon='+pandaread[1], 
                     'obslat':'obslat='+pandaread[2], 'obsn':'obsn='+str(1), 
                     'obsid':'obsid='+str(aidi[e]), 'obslev':'obslev='+str(1), 'obs':pandaread[3],
                     'obserr':pandaread[4]}
        pandawrite = pd.DataFrame(datawrite)
        pandawrite.to_csv(filewrite,sep = ',', index=False, header=False, mode='a')
            
        
        

# proc = subprocess.Popen([fortranexec, entrada, sortida1, sortida2], shell=True, stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
# stdout,stderr=proc.communicate()
# exit_code=proc.wait()
# print (exit_code,stdout.read(),stderr.read())
# def compile_fortran(source, modulename, extra_args=''):
#     import tempfile
#     import sys
#     import numpy.f2py
#     from numpy.distutils.exec_command import exec_command
    
#     # try:
#         # f = tempfile.NamedTemporaryFile(suffix='.f90')
#         # f.write(source)
#         # f.flush()

#     args = ' -c -m {} {} {}'.format(modulename, source, extra_args)
#     command = '{} -c "import numpy.f2py as f2py2e;f2py2e.main()" {}'.format(sys.executable, args)
#     status, output = exec_command(command)
#     return status, output, command
#     # finally:
#         # f.close()

# status, output, command = compile_fortran(sourcecode, 'rotar')
